#!/usr/bin/python
# -*- coding: utf-8 -*-
# vim: tabstop=4 expandtab shiftwidth=4
"""
Import files from given directories to your photo folder.
Optionally unmounts source media after successfull import.
"""
__revision__ = "$Id$"

import os
import sys
import stat
import shutil
import logging
import optparse
import subprocess
from . import filelist as exiflow_filelist


def run(argv, callback=None):
    """
    Take an equivalent of sys.argv[1:] and optionally a callable.
    Parse options, import files and optionally call the callable on every
    processed file with 3 arguments: filename, newname, percentage.
    If the callable returns True, stop the processing.
    """
    # Parse command line.
    parser = optparse.OptionParser()
    parser.add_option("-m", "--mount", dest="mount",
                      help="Mountpoint of directory to import. Corresponds"
                           " to %m in the gnome-volume-manager config dialog.")
    parser.add_option("-t", "--target", dest="target",
                      help="Target directory. A subdirectory will be created"
                           " in this directory.")
    parser.add_option("-d", "--device", dest="device",
                      help="(Ignored for backwards compatibility.)")
    parser.add_option("-v", "--verbose", action="store_true", dest="verbose",
                      help="Be verbose.")
    options, args = parser.parse_args(argv)
    logging.basicConfig(format="%(module)s: %(message)s")
    if options.verbose:
        logging.getLogger().setLevel(logging.INFO)
    logger = logging.getLogger("exiimport")
    if len(args) > 0 or not options.mount or not options.target:
        return "Wrong syntax.\n" + parser.format_help()

    # Build file list whithout skipping unknown files
    filelist = exiflow_filelist.Filelist([])
    filelist.process_unknown_types()
    filelist.add_files([options.mount])

    # Cry if we found no images
    if filelist.get_filecount() == 0:
        return "No files to import, sorry."

    # Create targetdir
    targetdir = os.path.join(options.target, filelist.get_daterange())
    # TODO: find a better solution than just appendings "+" chars.
    while os.path.exists(targetdir):
        targetdir += "+"
    os.makedirs(targetdir)
    logger.warning("Importing %s MB in %s files to %s",
                   filelist.get_fullsize() / 1024 / 1024,
                   filelist.get_filecount(), targetdir)

    # Copy files
    for filename, percentage in filelist:
        logger.info("%3s%% %s", percentage, filename)
        if callable(callback):
            if callback("", os.path.join(targetdir,
                                         os.path.basename(filename)),
                        percentage, keep_original=True):
                break

        shutil.copy2(filename, targetdir)
        os.chmod(os.path.join(targetdir, os.path.basename(filename)),
                 stat.S_IMODE(0644))

    # Unmount card
    mount = subprocess.Popen("mount", stdout=subprocess.PIPE)
    for line in mount.communicate()[0].splitlines():
        # Example line:
        # /dev/sdc1 on /media/NIKON D70 type vfat (rw,nosuid,n[...]
        line_parts = line.split(" type", 1)[0].split(None, 2)
        if (len(line_parts) == 3 and line_parts[1] == "on" and
           os.path.realpath(options.mount) == os.path.realpath(line_parts[2])):
            logger.warn("Trying to unmount %s.", line_parts[0])
            subprocess.call(["umount", line_parts[0]])


if __name__ == "__main__":
    sys.exit(run(sys.argv[1:]))



#!/usr/bin/python
# -*- coding: utf-8 -*-
# vim: tabstop=4 expandtab shiftwidth=4
"""
A set of functions to handle config files.
They look for existance of ~/.exiflow/<classname>.cfg and will create it if
it doesn't exist. Afterwards a configparser object of this file and
/etc/exiflow/<classname>.cfg is returned.
"""
__revision__ = "$Id: "

import os
import logging
import ConfigParser

global_config_dir = "/etc/exiflow"
local_config_dir = os.path.expanduser('~/.exiflow')

# Defaults
__default_contents = {"settings": """# settings.cfg
# This file defines some values to tell exiflow which types of files
# we want to process and which files and dirs we always want to skip.
# The only valid section here is [all].

[all]
image_extensions = .jpg .nef .raw .crw .tif
unwantend_files = nikon001.dsc
unwantend_dirs = .comments
""",
                      "cameras": """# cameras.cfg
# This file defines values to be used as filename parts and converters to be
# used for conversion of RAW file formats to JPEG.. An example filename:
#
# 20080718-n005966-sb000.jpg
#
# In this case cam_id was "n00" and artist_initials was "sb".
#
# While it is possible to only have an [all] section in this file, we recommend
# using camera specific sections for at least the cam_id to generate unique and
# meaningfull filenames.

#[all]
#cam_id = 000
#artist_initials = xy

#[NIKON Dxx]
#cam_id = n00
#artist_initials = yz
#raw_extension = .nef
#raw_converter = LANG=C ufraw-batch --gamma=0.45 --saturation=1.0 --exposure=0.0 --black-point=0 --interpolation=ahd --compression=90 --noexif --wb=camera --curve=linear --linearity=0.02 --clip=film --restore=lch --out-type=jpeg

#[HP PhotoSmart 715]
#cam_id = hp0

#[C2040Z]
#cam_id = o00
""",
                      "exif": """# exif.cfg
# This file contains EXIF information to be inserted into images.
# Configure global values in the [all] section and camera specific values
# in a section for the camera model. It is also possible to create sections
# with arbitrary names and supply these names later on the command line.
# All supported tags of ExifTool are allowed as keys - so it's possible to set
# or unset (just use "keyword =") specific tags with exiperson.

#[all]
#Artist = Arthur Dent
#Contact = adent@example.com
#Copyright = (c)2008 adent@example.com
#CopyrightNotice = All rights reserved, no redistribution without written permission.
#Credit = adent@example.com
# ATTENTION: You will want to set UserComment to an empty string even if you
# do not use it. It will prevent Exiftool from setting it to the word "ASCII".
#UserComment =

#[custom]
#artist = I. M. Weasel

#[NIKON Dxx]
#artist = I. R. Baboon

#[GT-I9100]
#ImageUniqueID =

"""}

# ConfigParser Caches
__cache = {}
__stats = {}


def __stat(filename):
    """
    Return a stat object or None if "filename" doesn't exist.
    """
    try:
        return os.stat(filename)
    except OSError:
        return None


def parse(configname):
    """
    Handle cached access to <configname>.cfg.
    Return a configparser object and a list of processed files.
    """
    logger = logging.getLogger("configfile.parse")
    local_config = os.path.join(local_config_dir, configname + ".cfg")
    global_config = os.path.join(global_config_dir, configname + ".cfg")
    if (not configname in __cache
            or __stats.get(local_config, None) != __stat(local_config)
            or __stats.get(global_config, None) != __stat(global_config)):
        if not os.path.exists(local_config):
            if not os.path.isdir(local_config_dir):
                os.makedirs(local_config_dir)
            file(local_config, "w").write(__default_contents[configname])
            logger.warning("Created example %s.", local_config)
        __stats[local_config] = __stat(local_config)
        __stats[global_config] = __stat(global_config)
        config = ConfigParser.ConfigParser()
        read_files = config.read([global_config, local_config])
        logger.info("Read %s config files: %s",
                    configname, " ".join(read_files))
        __cache[configname] = config
    return __cache[configname]


def get_options(configname, section, options):
    """
    Try to get options from section "section" from configfile "configname".
    If section doesn't exist or options don't exist in it, fall back to
    section "all". If that doesn't succed either, call append() and return
    empty values.
    """
    config = parse(configname)
    values = []
    must_append = False
    for option in options:
        if config.has_section(section) and config.has_option(section, option):
            values.append(config.get(section, option))
        elif config.has_section("all") and config.has_option("all", option):
            values.append(config.get("all", option))
        else:
            must_append = True
            values.append("")
    if must_append:
        append(configname, section, options)
    return values


def append(configname, section, options):
    """
    Append a commented section with options to "configname".cfg
    """
    logger = logging.getLogger("configfile.append")
    configfile = os.path.join(local_config_dir, configname + ".cfg")
    string_to_append = "\n#[%s]\n#%s = \n" % (section, " = \n#".join(options))
    if string_to_append in "".join(file(configfile, "r").readlines()):
        logger.warning("Commented section [%s] found in %s\nPlease edit!\n",
                       section, configfile)
    else:
        logger.warning("Adding commented section [%s] to %s\nPlease edit!\n",
                       section, configfile)
        file(configfile, "a").write(string_to_append)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: tabstop=4 expandtab shiftwidth=4
"""
Unit tests for exif.py
"""
__revision__ = "$Id$"


import shutil
import os
import sys
import tempfile
import unittest

import NikonD70
sys.path.append(os.path.dirname(sys.path[0]))
import exiflow.exif


class TestExif(unittest.TestCase):
    """ Tests for exif.py """

    def setUp(self):
        """ Create a directory with an image. """
        data_dir = os.path.join(os.path.dirname(__file__), "testdata")
        self.__fields = {"Keywords": u"foo, bar, thingamabob",
                         "Description": u"Hello world!"}
        self.__tempdir = tempfile.mkdtemp()
        self.__virginjpeg = os.path.join(self.__tempdir, "python.jpg")
        shutil.copy(os.path.join(data_dir, "python.jpg"), self.__tempdir)
        self.__d70jpeg = os.path.join(self.__tempdir, "NikonD70.jpg")
        shutil.copy(os.path.join(data_dir, "NikonD70.jpg"), self.__tempdir)
        self.__d70jpeg_imgdesc_latin1 = os.path.join(self.__tempdir,
            "NikonD70_ImageDescription_2rows_latin1.jpg")
        shutil.copy(os.path.join(data_dir,
            "NikonD70_ImageDescription_2rows_latin1.jpg"), self.__tempdir)
        self.__d70jpeg_imgdesc_utf8 = os.path.join(self.__tempdir,
            "NikonD70_ImageDescription_2rows_utf8.jpg")
        shutil.copy(os.path.join(data_dir,
            "NikonD70_ImageDescription_2rows_utf8.jpg"), self.__tempdir)
        self.__emptyfile = os.path.join(self.__tempdir, "emptyfile.jpg")
        file(self.__emptyfile, 'w')
        self.__nosuchfile = os.path.join(self.__tempdir, "nosuchfile.jpg")

    def tearDown(self):
        """ Clean up. """
        shutil.rmtree(self.__tempdir)

    def test_read_exif_without_image(self):
        """ Tests for read_exif() with invalid filename """
        exif = exiflow.exif.Exif(self.__nosuchfile)
        self.failUnlessRaises(IOError, exif.read_exif)

    def test_read_exif_with_corrupt_image(self):
        """ Tests for read_exif() with invalid image content """
        exif = exiflow.exif.Exif(self.__emptyfile)
        self.failUnlessRaises(IOError, exif.read_exif)

    def test_read_exif(self):
        """ Tests for read_exif() with valid image """
        exif = exiflow.exif.Exif(self.__d70jpeg)
        self.failUnless(isinstance(exif, exiflow.exif.Exif))
        exif.read_exif()
        for field in NikonD70.fields:
            self.failUnless(field in exif.fields, field + " not in fields")
            self.failUnlessEqual(exif.fields[field], NikonD70.fields[field])

    def test_read_exif_imagedescription(self):
        """ Tests for read_exif() with valid image """
        exif = exiflow.exif.Exif(self.__d70jpeg_imgdesc_utf8)
        self.failUnless(isinstance(exif, exiflow.exif.Exif))
        exif.read_exif()
        self.failUnlessEqual(exif.fields["ImageDescription"],
                             u"ImageDescription first row äöüß\n" +
                             u"ImageDescription second row ÄÖÜß\n")

    def test_read_exif_imagedescription_decode(self):
        """ Tests for read_exif() with valid image """
        exif = exiflow.exif.Exif(self.__d70jpeg_imgdesc_latin1)
        self.failUnless(isinstance(exif, exiflow.exif.Exif))
        exif.read_exif()
        self.failUnlessEqual(exif.fields["ImageDescription"],
                             u"ImageDescription first row äöüß\n" +
                             u"ImageDescription second row ÄÖÜß\n")

    def test_write_exif_without_image(self):
        """ Test for write_exif() with invalid filename """
        exif = exiflow.exif.Exif(self.__nosuchfile)
        exif.fields = self.__fields
        self.failUnlessRaises(IOError, exif.write_exif)

    def test_write_exif_with_corrupt_image(self):
        """ Test for write_exif() with invalid image content """
        exif = exiflow.exif.Exif(self.__emptyfile)
        exif.fields = self.__fields
        self.failUnlessRaises(IOError, exif.write_exif)

    def test_write_exif(self):
        """ Test for write_exif() with valid image """
        exif = exiflow.exif.Exif(self.__d70jpeg)
        exif.fields = self.__fields.copy()
        self.failUnlessEqual(exif.write_exif(), 0)
        exif.read_exif()
        for field in self.__fields:
            self.failUnlessEqual(exif.fields[field], self.__fields[field])

    def test_update_exif_without_image(self):
        """ Test for update_exif() with invalid filename """
        exif = exiflow.exif.Exif(self.__nosuchfile)
        self.failUnlessRaises(IOError, exif.update_exif, self.__d70jpeg)
        exif = exiflow.exif.Exif(self.__d70jpeg)
        self.failUnlessRaises(IOError, exif.update_exif, self.__nosuchfile)

    def test_update_exif_with_corrupt_image(self):
        """ Test for update_exif() with invalid image content """
        exif = exiflow.exif.Exif(self.__emptyfile)
        self.failUnlessRaises(IOError, exif.update_exif, self.__d70jpeg)
        exif = exiflow.exif.Exif(self.__d70jpeg)
        self.failUnlessRaises(IOError, exif.update_exif, self.__emptyfile)

    def test_update_exif(self):
        """ Test for update_exif() with valid image """
        exif = exiflow.exif.Exif(self.__virginjpeg)
        self.failUnlessEqual(exif.update_exif(self.__d70jpeg), 0)
        exif.read_exif()
        for field in NikonD70.fields:
            self.failUnless(field in exif.fields, field + " not in fields")
            self.failUnlessEqual(exif.fields[field], NikonD70.fields[field])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: tabstop=4 expandtab shiftwidth=4
"""
Unit tests for exiimport.py
"""
__revision__ = "$Id$"


import shutil
import os
import sys
import tempfile
import time
import unittest

import NikonD70
sys.path.append(os.path.dirname(sys.path[0]))
import exiflow.configfile
import exiflow.exiimport


class TestImport(unittest.TestCase):
    """ Tests for exiimport.py """

    def setUp(self):
        """ Create a directory with an image. """
        self.__data_dir = os.path.join(os.path.dirname(__file__), "testdata")
        self.__sourcedir = tempfile.mkdtemp()
        self.__targetdir = tempfile.mkdtemp()
        self.__configdir = tempfile.mkdtemp()
        exiflow.configfile.global_config_dir = self.__configdir
        exiflow.configfile.local_config_dir = self.__configdir

    def tearDown(self):
        """ Clean up. """
        shutil.rmtree(self.__configdir)
        shutil.rmtree(self.__sourcedir)
        shutil.rmtree(self.__targetdir)

    def test_run(self):
        """ Test run(). """
        shutil.copyfile(os.path.join(self.__data_dir, "NikonD70.jpg"),
                        os.path.join(self.__sourcedir, "DSC0001.jpg"))
        shutil.copyfile(os.path.join(self.__data_dir, "NikonD70.jpg"),
                        os.path.join(self.__sourcedir, "DSC0002.jpg"))
        exiflow.exiimport.run(argv=["--mount", self.__sourcedir,
                                    "--target", self.__targetdir,
                                    "--verbose"])
        self.assertEqual(os.listdir(self.__sourcedir),
                         os.listdir(os.path.join(self.__targetdir,
                                             os.listdir(self.__targetdir)[0])))
        # Run twice and check if another directory have been created.
        exiflow.exiimport.run(argv=["--mount", self.__sourcedir,
                                    "--target", self.__targetdir])
        self.assertEqual(len(os.listdir(self.__targetdir)), 2)

    def test_run__syntax_error(self):
        """ Test run() with wrong syntax. """
        result = exiflow.exiimport.run(argv=["--target", self.__targetdir,
                                       "bla"])
        self.assertTrue(result)

    def test_run__no_files(self):
        """ Test run() with empty source dir. """
        result = exiflow.exiimport.run(argv=["--mount", self.__sourcedir,
                                             "--target", self.__targetdir])
        self.assertTrue(result)

    def test_run__callback(self):
        """ Test run() with a callback function. """
        mylist = []

        def callback(*dummy_args, **dummy_kwargs):
            """ Just set callback_called to True. """
            mylist.append(True)
            return True

        shutil.copyfile(os.path.join(self.__data_dir, "NikonD70.jpg"),
                        os.path.join(self.__sourcedir, "DSC0001.jpg"))
        exiflow.exiimport.run(argv=["--mount", self.__sourcedir,
                                    "--target", self.__targetdir,
                                    "--verbose"], callback=callback)
        self.assertEqual(mylist, [True])




#!/usr/bin/python
# -*- coding: utf-8 -*-
# vim: tabstop=4 expandtab shiftwidth=4
"""
Rename a bunch of image files according to our holy file naming schema.

Given a camera that saves a file as:

dsc_1234.nef

we will rename that file to:

20050412-n001234-ur000.nef

At first there is 20050412. That's a date, telling us the
photo has been taken on 2005-04-12. This is determined by
looking it up in the image's EXIF information.

Then there is n001234. The "n00" part is read from a config
file as the three byte string to be put there for a given
camera model. In this case, the camera model information in
the EXIF header reads "Nikon D70", and the config section
for that model reads "n00", meaning an "n" as a model
indicator since I also own an HP camera for which I
configured "h00", and the "00" as a way to extend the
counting possibilities beyond 9999 pictures. Once the
camera switches from 9999 to 0000 I will change that string
to "n01". The "1234" part is just the numeric part of the
original filename.

At last there is ur000. "ur" are my initials; I have
simply configured "If it's a Nikon D70, the artist is me".
Of course there are possibilities to override that. The
"000" part is a revision number. This is an original,
untouched file, so it's revision is 000. An automatic
conversion to JPG would also have revision 000 since there
is no interaction and the files are still distinguishable
by their suffixes. Once I convert it with custom parameters
or do some kind of editing, I will save it as revision 100.
Another derivate of the original will get revision 200.
A derivate of revision 100 will get 110, a derivate of 110
will get 111 and another one will get 112. Got the idea?
Using this revision scheme lets you know about the basic
editing history (if there's any) by just looking at the
filename. If this is too complicated for your needs you
are free to use these three bytes in another way or to
leave them alone.

There's one exception regarding the initial "000" revision:
If the software detects the presence of a low quality JPG
accompanying a raw image, the raw file gets revision 000 as
usual, but the low quality file will get revision 00l so
that it can't be confused with an automatically converted
*000.jpg of high quality.
"""

__revision__ = "$Id$"

import os
import re
import sys
import time
import logging
import optparse
from . import exif
from . import filelist
from . import configfile


def get_exif_information(filename):
    """
    Read camera model and date from filename and return them.
    If The date isn't contained in EXIF, use the file's mtime.
    """
    exif_file = exif.Exif(filename)
    # read_exif may throw IOError. We leave the catching to our caller.
    exif_file.read_exif()
    model = exif_file.fields.get("Model", "all")
    date = exif_file.fields.get("DateTimeOriginal", "0")
    quality = exif_file.fields.get("Quality", "0")
    image_time = date
    if ":" in date:
        image_time = image_time[11:13] + image_time[14:16] + image_time[17:19]
        date = date[0:4] + date[5:7] + date[8:10]
    else:
        if date == "0":
            date = os.stat(filename).st_mtime
            image_time = date
        date = time.strftime("%Y%m%d", time.localtime(float(date)))
        image_time = time.strftime("%H%M%S", time.localtime(float(image_time)))
    return model, date, image_time, quality


def get_new_filename_parts(filename, my_filelist, quality):
    """
    Return a new name for filename according to our holy naming scheme.
    """
    leader, extension = os.path.splitext(filename)
    extension = extension.lower()
    number = "".join([char for char in leader[-4:] if char.isdigit()])
    revision = "000"
    # Look for high quality versions of this image. This is the case if we are
    # a .jpg, more than one file exists with the same prefix and the quality is
    # not "fine" or "high".
    if extension == ".jpg":
        versions = [vers[0] for vers in my_filelist
                    if vers[0].startswith(leader)]
        if len(versions) > 1:
            if quality.lower() not in ("fine", "high"):
                revision = "00l"
    if not number:
        raise IOError("Can't find a number in " + filename)
    return number.zfill(4), revision, extension


def rename_file(filename, my_filelist, with_time, cam_id=None,
                artist_initials=None):
    """
    Rename filename and return the newly generated name without dir.
    """
    logger = logging.getLogger("exirename.rename_file")
    match = re.match("^(\\d{8})(-(\\d{6}))?-(.{3})(\\d{4})-"
                     "((.{2})(.{3}))(\\.[^.]*)$",
                     os.path.basename(filename))
    # differentiate between getting values from existing filenames or getting
    # initial values from exif and config file
    if match:
        date = match.group(1)
        image_time = match.group(3)
        if cam_id is None:
            cam_id = match.group(4)
        number = match.group(5)
        if artist_initials is None:
            artist_initials = match.group(7)
        revision = match.group(8)
        extension = match.group(9)
        if with_time:
            if not image_time:
                image_time = get_exif_information(filename)[2]
            date += "-" + image_time
    else:
        model, date, image_time, quality = get_exif_information(filename)
        if with_time:
            date += "-" + image_time
        if cam_id is None or artist_initials is None:
            new_cam_id, new_artist_initials = configfile.get_options(
                "cameras", model, ("cam_id", "artist_initials"))
            cam_id = cam_id or new_cam_id
            artist_initials = artist_initials or new_artist_initials
        number, revision, extension = get_new_filename_parts(filename,
                                                             my_filelist,
                                                             quality)

    if len(cam_id) != 3 or len(artist_initials) != 2:
        logger.warning("Either cam_id or artist_initials is missing or of "
                       "wrong length. cam_id should be 3 characters and is "
                       "currently set to '%s', artist_initials should be 2 "
                       "characters and is set to '%s'. "
                       "Skipping %s.", cam_id, artist_initials, filename)
        return os.path.basename(filename)

    newbasename = (date + "-" + cam_id + number + "-" + artist_initials +
                   revision + extension)
    newname = os.path.join(os.path.dirname(filename), newbasename)
    if filename == newname:
        raise IOError("Filename does not change.")
    elif os.path.exists(newname):
        raise IOError("Can't rename %s to %s, it already exists." % (filename,
                                                                     newname))
    os.rename(filename, newname)
    return newbasename


def run(argv, callback=None):
    """
    Take an equivalent of sys.argv[1:] and optionally a callable.
    Parse options, rename files and optionally call the callable on every
    processed file with 3 arguments: filename, newname, percentage.
    If the callable returns True, stop the processing.
    """
    parser = optparse.OptionParser(usage="usage: %prog [options] "
                                         "<files or dirs>")
    parser.add_option("--cam_id", "-c", dest="cam_id",
                      help="ID string for the camera model. Should normally be"
                           " three characters long.")
    parser.add_option("--artist_initials", "-a", dest="artist_initials",
                      help="Initials of the artist. Should be two characters"
                           " long.")
    parser.add_option("-t", "--with_time", action="store_true",
                      dest="with_time",
                      help="Create filenames containing the image time, for "
                           "example 20071231-235959-n001234-xy000.jpg instead "
                           "of 20071231-n001234-xy000.jpg .")
    parser.add_option("-v", "--verbose", action="store_true", dest="verbose",
                      help="Be verbose.")
    options, args = parser.parse_args(args=argv)

    logging.basicConfig(format="%(module)s: %(message)s")
    if options.verbose:
        logging.getLogger().setLevel(logging.INFO)
    logger = logging.getLogger("exirename")

    my_filelist = filelist.Filelist(args)
    for filename, percentage in my_filelist:
        try:
            newname = rename_file(filename, my_filelist, options.with_time,
                                  options.cam_id, options.artist_initials)
        except IOError, msg:
            newname = os.path.basename(filename)
            logger.error("Skipping %s:\n %s", filename, str(msg))
        logger.info("%3s%% %s -> %s", percentage, filename, newname)
        if callable(callback):
            if callback(filename,
                        os.path.join(os.path.dirname(filename), newname),
                        percentage):
                break


if __name__ == "__main__":
    run(sys.argv[1:])



#!/usr/bin/python
# -*- coding: utf-8 -*-
# vim: tabstop=4 expandtab shiftwidth=4
"""
Find groups of derived images and exchange EXIF information between them.

Example:
 These files are a raw image, a low quality version and two edited versions:
 20050914-n001235-jd000.nef
 20050914-n001235-jd00l.jpg
 20050914-n001235-jd100.jpg
 20050914-n001235-jd110.jpg
 The last one doesn't have EXIF informations. This tool will find that out and
 look at 20050914-n001235-jd100.jpg and 20050914-n001235-jd000.nef in this
 order to copy the EXIF header from.
"""
__revision__ = "$Id$"

import os
import re
import sys
import glob
import logging
import optparse
from . import exif
from . import filelist


def find_siblings(filename, prefix):
    """
    Try to find siblings for "filename" in the same directory and return them
    as a list of files reverse sorted by modification time.
    Return [] if no sibling is found.
    A file is considered a sibling if it's basename starts with "prefix".
    """
    mtimes = {}
    for otherfile in glob.glob(os.path.join(os.path.dirname(filename),
                                            prefix + "*")):
        extension = os.path.splitext(otherfile)[1]
        extension = extension.lower()

        if otherfile == filename:
            continue
        elif extension in (".ufraw", ".xmp"):
            continue
        else:
            mtimes[str(os.stat(otherfile).st_mtime) + otherfile] = otherfile
    return [mtimes[mtime] for mtime in sorted(mtimes, reverse=True)]


def assign_file(filename, prefix, force=False):
    """
    Process "filename", passing "prefix" to find_siblings().
    With force=True, force update even if EXIF is already present.
    """
    logger = logging.getLogger("exiassign.assign_file")
    exif_file = exif.Exif(filename)
    try:
        exif_file.read_exif()
    except IOError, msg:
        logger.warning(str(msg))
        return 1
    # Currently F-spot always tries to write "DateTimeOriginal", so we must
    # change the key to detect files with valid exif information.
    #if not force and exif_file.fields.has_key("DateTimeOriginal"):
    if not force and "Model" in exif_file.fields:
        logger.info("Skipping %s, it seems to contain EXIF data.", filename)
        return 0
    for sibling in find_siblings(filename, prefix):
        other_exif_file = exif.Exif(sibling)
        try:
            other_exif_file.read_exif()
        except IOError:
            continue
        other_exif_file.fields.update(exif_file.fields)
        if other_exif_file.fields != exif_file.fields:
            logger.info("Updating %s from %s.", filename, sibling)
            exif_file.update_exif(sibling)
            return 0
    logger.info("No sibling found for %s.", filename)
    return 0


def run(argv, callback=None):
    """
    Take an equivalent of sys.argv[1:] and optionally a callable.
    Parse options, assign relating files and gate meta information between
    them, and optionally call the callable on every processed file with
    3 arguments:
    filename, newname, percentage.
    If the callable returns True, stop the processing.
    """
    parser = optparse.OptionParser(usage="usage: %prog [options] "
                                         "<files or dirs>")
    parser.add_option("-f", "--force", action="store_true", dest="force",
                      help="Force update even if EXIF is already present. "
                           "The fields handled by these scripts are kept "
                           "anyway.")
    parser.add_option("-v", "--verbose", action="store_true", dest="verbose",
                      help="Be verbose.")
    options, args = parser.parse_args(argv)

    if len(args) == 0:
        parser.print_help()
        sys.exit(1)

    logging.basicConfig(format="%(module)s: %(message)s")
    if options.verbose:
        logging.getLogger().setLevel(logging.INFO)
    logger = logging.getLogger("exiassign")

    filename_re = re.compile("^(\\d{8}(-\\d{6})?-.{3}\\d{4}-)(.{5})\\.[^.]*$")

    for filename, percentage in filelist.Filelist(args):
        mymatch = filename_re.match(os.path.basename(filename))
        if mymatch:
            logger.info("%3s%% %s", percentage, filename)
            prefix = mymatch.groups()[0]
            assign_file(filename, prefix, options.force)
            if callable(callback):
                if callback(filename, filename, percentage):
                    break


if __name__ == "__main__":
    run(sys.argv[1:])



#!/usr/bin/python
# -*- coding: utf-8 -*-
# vim: tabstop=4 expandtab shiftwidth=4
"""
Automatically gate image meta information between EXIF
headers and Gthumb comment files.
"""
__revision__ = "$Id$"

import os
import sys
import logging
import optparse
from . import exif
from . import gthumb
from . import filelist


def autogate_gthumb(filename, myoptions):
    """
    Decide if we gate from gthumb to Exif or vice versa by looking at the
    time stamps of the comment and the Exif file.
    Run read and write functions accordingly.
    """
    logger = logging.getLogger("exigate.autogate_gthumb")
    filename = os.path.abspath(filename)
    gthumbfile = gthumb.Gthumb(filename)
    gthumbtimestamp = gthumbfile.get_mtime()
    filetimestamp = os.path.getmtime(filename)
    if gthumbtimestamp > filetimestamp:
        logger.info("Updating %s from comment file.", filename)
        try:
            gthumbfile.read()
        except IOError, msg:
            logger.error("Error reading gthumb comment for %s:\n%s",
                         filename, msg)
            return False
        exif_file = exif.Exif(filename)
        exif_file.fields = gthumbfile.fields
        try:
            exif_file.write_exif()
        except IOError, msg:
            logger.error("Error writing EXIF data to %s:\n%s", filename, msg)
            return False
        # TODO: Find out why we intruduced this line. Seems odd...
        #write_gthumb(filename, gthumb, myoptions.addfields,
        #             myoptions.template)
        # Maybe that way we wanted to update in addfields and template mode
        # even if the file's timestamp is older than the gthumb comment? Check
        # if that is needed and if so, if we can do it conditionally!
        gthumbfile.set_mtime(filetimestamp)
        os.utime(filename, (filetimestamp, filetimestamp))
    elif (filetimestamp > gthumbtimestamp
          or myoptions.addfields
          or myoptions.template):
        logger.info("Updating comment file from %s.", filename)
        exif_file = exif.Exif(filename)
        try:
            exif_file.read_exif()
        except IOError, message:
            logger.info("Skipping %s: %s", filename, message)
            return 1
        gthumbfile.fields = exif_file.fields
        gthumbfile.write(myoptions.addfields, myoptions.template)
        gthumbfile.set_mtime(filetimestamp)
    else:
        logger.info("%s is in sync with comment file.", filename)
    return True


def run(argv, callback=None):
    """
    Take an equivalent of sys.argv[1:] and optionally a callable.
    Parse options, gate meta information and optionally call the callable on
    every processed file with 3 arguments: filename, newname, percentage.
    If the callable returns True, stop the processing.
    """
    parser = optparse.OptionParser(usage="usage: %prog [options] "
                                         "<files or dirs>")
    parser.add_option("--gthumb", action="store_true", dest="gthumb",
                      help="Gateway to/from gthumb comment files."
                           "This is the default.")
    parser.add_option("--additional-fields", "-a", action="store_true",
                      dest="addfields",
                      help="When gating from Exif to gthumb, combine "
                           "additional Exif fields into the comment.")
    parser.add_option("--template", "-t", action="store_true", dest="template",
                      help="Like --additional-fields, but also combine empty "
                           "fields as templates into the comment.")
    # TODO:
    #parser.add_option("--merge", action="store_true", dest="merge",
    #                  help="Merge data instead of just using the newest "
    #                       "version to overwrite the older one.")
    parser.add_option("--cleanup", "-c", action="store_true", dest="cleanup",
                      help="The opposite of --additional-fields, that is, "
                           "remove additional fields from gthumb comments.")
    parser.add_option("-v", "--verbose", action="store_true", dest="verbose",
                      help="Be verbose.")
    options, args = parser.parse_args(argv)

    logging.basicConfig(format="%(module)s: %(message)s")
    if options.verbose:
        logging.getLogger().setLevel(logging.INFO)
    logger = logging.getLogger("exigate")

    for filename, percentage in filelist.Filelist(args):
        if callable(callback):
            if callback(filename, filename, percentage):
                break
        logger.info("%3s%% %s", percentage, filename)
        autogate_gthumb(filename, options)
        if options.cleanup:
            log_prefix = "Leaving"
            if gthumb.Gthumb(filename).cleanup():
                log_prefix = "Cleaning"
            if options.verbose:
                logger.info("%s comment file of %s.", log_prefix, filename)


if __name__ == "__main__":
    run(sys.argv[1:])



#!/usr/bin/python
# -*- coding: utf-8 -*-
# vim: tabstop=4 expandtab shiftwidth=4
"""
Generate lists of files given as names or searched in directories and
provide an iterable interface to them.
"""
__revision__ = "$Id$"

import os
import time
import logging
from . import configfile


class Filelist(object):
    """
    An iterable class that hosts information about files

    given in it's contructor.
    """

    def __init__(self, pathes):
        """
        Create FileList object.
        Call add_files() with all arguments.
        If process_unknown_types is given, then no files will be skipped.
        """
        self._files = []
        self._skippedfiles = []
        self._filedates = {}
        self._filestats = {}
        self._fullsize = 0
        self._process_unknown_types = False
        for path in pathes:
            self.add_files([path])

    def add_files(self, pathes):
        """
        Add filenames found in pathes either to _files or _skippedfiles,
        depending on their extension and the value of _process_unknown_types.
        Return False if files are skipped or no files are processed,
        True otherwise.
        Raises IOError on access errors.
        """
        logger = logging.getLogger("filelist.add_files")
        settings = configfile.parse("settings")
        unwanted_dirs = settings.get("all", "unwantend_dirs").split()
        for path in pathes:
            if path.startswith("file:///"):
                path = path[7:]
            if os.path.isfile(path):
                self._add_file(path)
            elif os.path.isdir(path):
                for root, dirs, files in os.walk(path, True):
                    for unwanted_dir in unwanted_dirs:
                        if unwanted_dir in dirs:
                            logger.info("Skipping unwanted dir %s.",
                                        os.path.join(root, unwanted_dir))
                            dirs.remove(unwanted_dir)
                    for basefile in files:
                        self._add_file(os.path.join(root, basefile))
            else:
                logger.warning("%s is not a regular file or directory. "
                               "Skipping.", path)
        return bool(len(self._files)) and not bool(len(self._skippedfiles))

    def _add_file(self, filename):
        """
        Add filename and all it's data to the filelist.
        Return True if the file is added, False otherwise.
        """
        logger = logging.getLogger("filelist._add_file")
        settings = configfile.parse("settings")
        image_extensions = settings.get("all", "image_extensions").split()
        unwanted_files = settings.get("all", "unwantend_files").split()
        basefilename = os.path.basename(filename).lower()
        if ((self._process_unknown_types
             or os.path.splitext(basefilename)[1] in image_extensions)
                and not basefilename in unwanted_files):
            self._files.append(filename)
            filestat = os.stat(filename)
            self._filestats[filename] = filestat
            self._fullsize += filestat.st_size
            self._filedates[filename] = \
                time.strftime("%Y-%m-%d", time.localtime(filestat.st_mtime))
            return True
        else:
            logger.info("Skipping %s.", filename)
            self._skippedfiles.append(filename)
            return False

    def process_unknown_types(self):
        """
        Set _process_unknown_types to True, meaning that following method calls
        should work on every file except those listed in "unwanted_files" in
        the config file. Typical usage will be to call the constructor without
        arguments, then call this method, and then call add_files().
        """
        self._process_unknown_types = True

    def __iter__(self):
        """
        Iterate through filelist and update percentage.
        Returns (filename, percentage) pairs on each iteration.
        """
        size_so_far = 0
        for filename in self._files:
            size_so_far += self._filestats[filename].st_size
            yield(filename, 100 * size_so_far / self._fullsize)

    def get_daterange(self):
        """
        Get the date range of the files as a string in the form
        YYYY-MM-DD_-_YYYY-MM-DD   or  YYYY-MM-DD  if only one date available.
        """
        dates = sorted(set(self._filedates.values()))
        range_string = dates[0]
        if len(dates) > 1:
            range_string += "_-_" + dates.pop()
        return range_string

    def get_date(self, filename):
        """
        Get modification date for filename.
        """
        return self._filedates[filename]

    def get_fullsize(self):
        """
        Get cumulated bytes of files.
        """
        return self._fullsize

    def get_filecount(self):
        """
        Get number of files.
        """
        return len(self._files)

    def get_files(self):
        """
        Get al list of all files.
        """
        return self._files



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Captured metadata of NikonD70.jpg .
Make sure to ignore "Directory" and "FileModifyDate" when using this for
comparison purposes..
"""
__revision__ = "$Id$"

fields = {'AFAperture': u'4.5',
          'AFAreaMode': u'Single Area',
          'AFPoint': u'Center',
          'AFPointsInFocus': u'Center',
          'Aperture': u'9.0',
          #'AutoBracketRelease': u'None',
          'BitsPerSample': u'8',
          'BlueBalance': u'1.636719',
          'CFAPattern': u'[Blue,Green][Green,Red]',
          'CircleOfConfusion': u'0.020 mm',
          'ColorComponents': u'3',
          'ColorHue': u'Mode1a',
          'ColorSpace': u'Uncalibrated',
          'ComponentsConfiguration': u'Y, Cb, Cr, -',
          'CompressedBitsPerPixel': u'4',
          'Compression': u'JPEG (old-style)',
          'Contrast': u'Normal',
          'CreateDate': u'1080636226',
          'CustomRendered': u'Normal',
          'DOF': u'4.70 m (3.85 - 8.55)',
          'DateTimeOriginal': u'1080636226',
          'DigitalZoomRatio': u'1',
          'EffectiveMaxAperture': u'4.5',
          'EncodingProcess': u'Baseline DCT, Huffman coding',
          'ExifByteOrder': u'Big-endian (Motorola, MM)',
          'ExifVersion': u'0221',
          'ExitPupilPosition': u'89.0 mm',
          'ExposureBracketValue': u'0',
          'ExposureCompensation': u'0',
          'ExposureDifference': u'0',
          'ExposureMode': u'Auto',
          'ExposureProgram': u'Program AE',
          'ExposureTime': u'1/250',
          'ExternalFlashFirmware': u'n/a',
          'ExternalFlashFlags': u'(none)',
          'FNumber': u'9.0',
          'FOV': u'26.7 deg (2.52 m)',
          'Flash': u'No Flash',
          'FlashCommanderMode': u'Off',
          'FlashControlMode': u'Off',
          'FlashExposureBracketValue': u'0.0',
          'FlashExposureComp': u'0',
          'FlashGroupAControlMode': u'Off',
          #'FlashGroupAExposureComp': u'0',
          'FlashGroupBControlMode': u'Off',
          #'FlashGroupBExposureComp': u'0',
          'FlashInfoVersion': u'0100',
          'FlashMode': u'Did Not Fire',
          'FlashSetting': u'Normal',
          'FlashType': u'',
          'FlashpixVersion': u'0100',
          'FocalLength': u'50.0 mm',
          'FocalLength35efl': u'50.0 mm (35 mm equivalent: 75.0 mm)',
          'FocalLengthIn35mmFormat': u'75 mm',
          'FocusDistance': u'5.31 m',
          'FocusMode': u'AF-S',
          'FocusPosition': u'0xa1',
          'GainControl': u'None',
          'Gamma': u'2.2',
          'HueAdjustment': u'0',
          'HyperfocalDistance': u'13.87 m',
          'ISO': u'200',
          'ISOSetting': u'200',
          'ImageOptimization': u'Normal',
          'Lens': u'18-70mm f/3.5-4.5',
          'LensDataVersion': u'0101',
          'LensFStops': u'5.33',
          'LensID': u'AF-S DX Zoom-Nikkor 18-70mm f/3.5-4.5G IF-ED',
          'LensIDNumber': u'127',
          'LensSpec': u'18-70mm f/3.5-4.5 G',
          'LensType': u'G',
          'LightSource': u'Natural',
          'LightValue': u'13.3',
          'MCUVersion': u'132',
          'MIMEType': u'image/jpeg',
          'Make': u'NIKON CORPORATION',
          'MakerNoteVersion': u'2.10',
          'MaxApertureAtMaxFocal': u'4.5',
          'MaxApertureAtMinFocal': u'3.6',
          'MaxApertureValue': u'4.4',
          'MaxFocalLength': u'71.3 mm',
          'MeteringMode': u'Multi-segment',
          'MinFocalLength': u'18.3 mm',
          'Model': u'NIKON D70',
          'ModifyDate': u'1080636226',
          'NoiseReduction': u'Off',
          'PrimaryChromaticities': u'0.64 0.33 0.21 0.71 0.15 0.06',
          'ProgramShift': u'0',
          'Quality': u'Fine',
          'RedBalance': u'2.015625',
          'ResolutionUnit': u'inches',
          'Saturation': u'Normal',
          'ScaleFactor35efl': u'1.5',
          'SceneCaptureType': u'Standard',
          'SceneType': u'Directly photographed',
          'SensingMethod': u'One-chip color area',
          'SensorPixelSize': u'7.8 x 7.8 um',
          'SerialNumber': u'No= 100005e5',
          'Sharpness': u'Normal',
          'ShootingMode': u'Single-Frame',
          'ShutterCount': u'1193',
          'ShutterSpeed': u'1/250',
          'Software': u'Ver.1.01',
          'SubSecCreateDate': u'1080636226',
          'SubSecDateTimeOriginal': u'1080636226',
          'SubSecModifyDate': u'1080636226',
          'SubSecTime': u'00',
          'SubSecTimeDigitized': u'00',
          'SubSecTimeOriginal': u'00',
          'SubjectDistanceRange': u'Unknown',
          'ThumbnailImage':
                      u'(Binary data 8930 bytes, use -b option to extract)',
          'ThumbnailLength': u'8930',
          'ToneComp': u'Auto',
          'UserComment': u'',
          'VariProgram': u'',
          'WB_RGBGLevels': u'516 256 419 256',
          'WhiteBalance': u'Auto',
          'WhiteBalanceFineTune': u'0',
          'WhitePoint': u'0.313 0.329',
          'XResolution': u'300',
          'YResolution': u'300'}



#!/usr/bin/python
# -*- coding: utf-8 -*-
# vim: tabstop=4 expandtab shiftwidth=4
"""
Convert any RAW images into JPEG images.

The raw file extensions as well as the converters to call
are read from the config file "cameras.cfg" using the keys
"raw_extension" and "raw_converter". As alway, the [all]
section may contain general defaults used for all cameras
and camera specific sections may contain values for a given
camera model.

An example section would look like this:

[NIKON D70]
raw_extension = .nef
raw_converter = ufraw-batch --clip --out-type=jpeg

Given this, any files produced by a Nikon D70 that end in
".nef" are converted using this artificial ufraw call.

The raw converter is expected to write a file of the same
prefix as the input file with an extension of ".jpg".
For example, a file "test.nef" should result in "test.jpg".
"""

__revision__ = "$Id$"

import os
import re
import sys
import logging
import optparse
import subprocess
from . import exif
from . import filelist
from . import configfile


def convert_file(filename):
    """
    Convert filename and return the newly generated name without dir.
    """
    logger = logging.getLogger("exiconvert.convert_file")
    basename = os.path.basename(filename)
    # Sanity check - no sense in trying to convert jpeg to jpeg
    if basename.lower().endswith(".jpg"):
        logger.debug("%s is a Jpeg file, skipping.", basename)
        return basename

    exif_file = exif.Exif(filename)
    # read_exif may throw IOError. We leave the catching to our caller.
    exif_file.read_exif()
    model = exif_file.fields.get("Model", "all")

    raw_extension, raw_converter = \
        configfile.get_options("cameras", model,
                               ("raw_extension", "raw_converter"))

    if raw_extension == "":
        logger.warning("No raw_extension configured, skipping %s", basename)
        return basename

    if not filename.endswith(raw_extension):
        logger.info("%s doesn't match, skipping.", filename)
        return basename

    if raw_converter == "":
        logger.warning("No raw_converter configured, skipping %s", basename)
        return basename

    newname = os.path.splitext(filename)[0] + ".jpg"
    if os.path.exists(newname):
        logger.info("%s already exists, skipping %s.", newname, basename)
    else:
        command = raw_converter + " " + filename
        process = subprocess.Popen(command, shell=True, stdout=subprocess.PIPE,
                                   stderr=subprocess.PIPE)
        for line in process.stderr:
            logger.warning(line)
        for line in process.stdout:
            logger.info(line)
        if process.wait() > 0:
            logger.error("Converter invocation returned an error:\n%s",
                         command)
        else:
            if not os.path.exists(newname):
                logger.error("Converter returned 0, but %s is absent!",
                             newname)
            else:
                return os.path.basename(newname)
    return basename


def run(argv, callback=None):
    """
    Take an equivalent of sys.argv[1:] and optionally a callable.
    Parse options, convert convertible files and optionally call the callable
    on every processed file with 4 arguments: filename, newname, percentage,
    keep_original=true. The latter option means "The new file is an addition,
    not a replacement". If the callable returns True, stop the processing.
    """
    parser = optparse.OptionParser(usage="usage: %prog [options] "
                                         "<files or dirs>")
    parser.add_option("-r", "--remove-lqjpeg", action="store_true",
                      dest="remove_lqjpeg",
                      help="removes the low quality jpeg (...00l.jpg) "
                            "example yyyymmdd-a00000-xy00l.jpg")
    parser.add_option("-v", "--verbose", action="store_true", dest="verbose",
                      help="Be verbose.")
    options, args = parser.parse_args(args=argv)

    logging.basicConfig(format="%(module)s: %(message)s")
    if options.verbose:
        logging.getLogger().setLevel(logging.INFO)
    logger = logging.getLogger("exiconvert")

    filename_re = re.compile("^(\\d{8}(-\\d{6})?-.{3}\\d{4}-.{2})000\\.jpg$")
    for filename, percentage in filelist.Filelist(args):
        callback_filename = None
        logger.info("%3s%% %s", percentage, filename)
        if callable(callback):
            if callback(filename, filename, percentage):
                break
        try:
            newname = convert_file(filename)
        except IOError, msg:
            newname = os.path.basename(filename)
            logger.error("Skipping %s:\n%s\n", filename, msg)
        if options.remove_lqjpeg:
            keep_original = False
            mymatch = filename_re.match(newname)
            if mymatch:
                lqname = os.path.join(os.path.dirname(filename),
                                      mymatch.group(1) + "00l.jpg")
                if os.path.exists(lqname):
                    try:
                        os.remove(lqname)
                        callback_filename = lqname
                    except IOError, msg:
                        logger.error("Skipping remove of %s:\n%s\n",
                                     lqname, msg)
        else:
            callback_filename = filename
            keep_original = True

        if callback_filename is not None and callable(callback):
            if callback(callback_filename,
                        os.path.join(os.path.dirname(filename), newname),
                        percentage, keep_original=keep_original):
                break


if __name__ == "__main__":
    run(sys.argv[1:])



#!/usr/bin/python
# -*- coding: UTF-8 -*-
# vim: tabstop=4 expandtab shiftwidth=4
"""
A nice PyGTK GUI for the exiflow tool collection.
"""
__revision__ = "$Id$"

import os
import sys
import logging
import optparse
import pygtk
import shutil
import tempfile
pygtk.require("2.0")
import gtk
import gtk.glade

from . import filelist
from . import exiassign
from . import exiconvert
from . import exigate
from . import exiimport
from . import exiperson
from . import exirename

gladefile = os.path.splitext(__file__)[0] + ".glade"


class WritableTextView(object):
    """
    Provide a file object that writes to a GTK textview.
    """
    def __init__(self, textview, color=None):
        """
        Construct the file object writing to textview, optionally using color.
        """
        self.textview = textview
        self.buffer = self.textview.get_buffer()
        self.tag_names = []
        tag_table = self.buffer.get_tag_table()
        if not tag_table.lookup("warning"):
            tag = gtk.TextTag("warning")
            tag.set_property("foreground", "red")
            tag.set_property("background", "yellow")
            tag_table.add(tag)
        if color:
            if not tag_table.lookup(color):
                tag = gtk.TextTag(color)
                tag.set_property("foreground", color)
                tag_table.add(tag)
            self.tag_names.append(color)

    def write(self, msg):
        """
        Output msg to the textview.
        """
        my_iter = self.buffer.get_end_iter()
        tag_names = self.tag_names[:]
        if msg.startswith("WARNING") or msg.startswith("ERROR"):
            tag_names.append("warning")
        self.buffer.insert_with_tags_by_name(my_iter, msg, *tag_names)
        self.textview.scroll_mark_onscreen(self.buffer.get_insert())

    def flush(self):
        """
        Imitate a buffer flush.
        Since a textview can't be flushed, do nothing.
        """
        pass


class Directorychooser1(object):
    """
    Create a window that allows the user to select a directory.
    """
    def __init__(self, parent=None, callback=None):
        """ Instantiate the chooser. """
        self.wtree = gtk.glade.XML(gladefile, "directorychooserdialog1")
        self.window = self.wtree.get_widget("directorychooserdialog1")
        dic = {}
        for key in dir(self.__class__):
            dic[key] = getattr(self, key)
        self.wtree.signal_autoconnect(dic)
        if parent:
            self.window.set_transient_for(parent)
        self.callback = callback
        self.window.show()

    def on_directorychooserdialog1_response(self, widget, data=None):
        """ Callback function for the chooser. """
        if data == gtk.RESPONSE_OK and callable(self.callback):
            self.callback(widget.get_filename())
        self.window.destroy()


class Filechooser1(object):
    """
    Create a window that allows the user to select files.
    """
    def __init__(self, parent=None, callback=None):
        """ Instantiate the chooser. """
        self.wtree = gtk.glade.XML(gladefile, "filechooserdialog1")
        self.window = self.wtree.get_widget("filechooserdialog1")
        dic = {}
        for key in dir(self.__class__):
            dic[key] = getattr(self, key)
        self.wtree.signal_autoconnect(dic)
        if parent:
            self.window.set_transient_for(parent)
        self.callback = callback
        self.window.show()

    def on_filechooserdialog1_response(self, widget, data=None):
        """ Callback function for the chooser. """
        if data == gtk.RESPONSE_OK and callable(self.callback):
            self.callback(widget.get_filenames())
        self.window.destroy()


class Window1(object):
    """
    The program's main window.
    """
    def __init__(self):
        """ Instantiate the main window. """
        self._cancelled = False
        self.wtree = gtk.glade.XML(gladefile, "mainwindow")
        self.window = self.wtree.get_widget("mainwindow")
        # Initialize treeview
        treeview = self.wtree.get_widget("treeview1")
        self.liststore = gtk.ListStore(str)
        treeview.set_model(self.liststore)
        text_cell = gtk.CellRendererText()
        text_column = gtk.TreeViewColumn("Filename")
        text_column.pack_start(text_cell, True)
        text_column.add_attribute(text_cell, "text", 0)
        #text_column.set_attributes(text_cell, markup=1)
        #wir müssen markup anschalten um den text später formatieren zu können
        treeview.append_column(text_column)
        dic = {}
        for key in dir(self.__class__):
            dic[key] = getattr(self, key)
        self.wtree.signal_autoconnect(dic)
        self.window.show()
        # Create TextView and use it
        sys.stdout = WritableTextView(self.wtree.get_widget("textview1"))
        sys.stderr = WritableTextView(self.wtree.get_widget("textview1"),
                                      "blue")
        stdlog = WritableTextView(self.wtree.get_widget("textview1"), "red")
        logging.basicConfig(format="%(module)s: %(message)s", stream=stdlog,
                            level=logging.INFO)
        self.batch_scripts = []
        self.batch_tmpdir = ""
        self.batch_target = ""
        self.window.connect("map_event", self.batch_run)

    def _make_sensitive(self, name):
        """ Set widget called name sensitive. """
        self.wtree.get_widget(name).set_sensitive(True)

    def _make_insensitive(self, name):
        """ Set widget called name insensitive. """
        self.wtree.get_widget(name).set_sensitive(False)

    def _is_active(self, name):
        """ Return True if widget "name" is activated, False otherwise. """
        return self.wtree.get_widget(name).get_active()

    def set_active(self, name):
        """ Activate button of widget "name". """
        self.wtree.get_widget(name).set_active(1)

    def set_text(self, name, text):
        """ Set text of widget "name" to text. """
        self.wtree.get_widget(name).set_text(text)

    def on_button_open_clicked(self, *dummy):
        """ Callback for the "open" button. """
        dummy = Filechooser1(self.window, self.set_filelist)

    def on_button_exiimport_browse_importdir_clicked(self, *dummy):
        """ Callback for the exiimport's "browse importdir" button. """
        dummy = Directorychooser1(self.window,
                                  self.wtree.get_widget(
                                      "exiimport_importdir_entry").set_text)

    def on_button_exiimport_browse_targetdir_clicked(self, *dummy):
        """ Callback for the exiimport's "browse targetdir" button. """
        dummy = Directorychooser1(self.window,
                                  self.wtree.get_widget(
                                      "exiimport_targetdir_entry").set_text)

    @staticmethod
    def on_info1_activate(*dummy):
        """ Callback for the "about" menu entry. """
        wtree = gtk.glade.XML(gladefile, "aboutdialog1")
        window = wtree.get_widget("aboutdialog1")
        window.show_all()
        window.run()
        window.hide()

    @staticmethod
    def on_mainwindow_destroy(*dummy):
        """ Callback for the window's close button. """
        gtk.main_quit()

    def set_filelist(self, files):
        """
        Put files into the filelist.
        Directories are extrapolated via the Filelist class.
        """
        logger = logging.getLogger("exigui.set_filelist")
        self.liststore.clear()
        for filename in filelist.Filelist(files).get_files():
            filename = os.path.abspath(filename)
            if os.path.exists(filename):
                self.liststore.append([filename])
            else:
                logger.warning("%s doesn't exist!", filename)

    def on_exirename_camid_auto_activate(self, *dummy):
        """ Callback for exirename's "auto cam_id" selection. """
        self._make_insensitive("exirename_cam_id_entry")

    def on_exirename_camid_custom_activate(self, *dummy):
        """ Callback for exirename's "custom cam_id" selection. """
        self._make_sensitive("exirename_cam_id_entry")

    def on_exirename_artist_auto_activate(self, *dummy):
        """ Callback for exirename's "auto artist" selection. """
        self._make_insensitive("exirename_artist_initials_entry")

    def on_exirename_artist_custom_activate(self, *dummy):
        """ Callback for exirename's "custom artist" selection. """
        self._make_sensitive("exirename_artist_initials_entry")

    def on_exiperson_section_auto_activate(self, *dummy):
        """ Callback for exirename's "auto section" selection. """
        self._make_insensitive("exiperson_section_entry")

    def on_exiperson_section_custom_activate(self, *dummy):
        """ Callback for exirename's "custom section" selection. """
        self._make_sensitive("exiperson_section_entry")

    def _progress_callback(self, filename, newname, percentage,
                           keep_original=False):
        """
        This callback is given as a callable to the main programs and is
        called after each processed file. filename and newname may of course
        be the same. If keep_original is True, add newname instead of replacing
        filename with it.
        Return self._cancelled which is True after Cancel has been pressed.
        """
        if filename != newname:
            if keep_original:
                self.liststore.append([newname])
            else:
                for rownum in range(0, len(self.liststore)):
                    if self.liststore[rownum][0] == filename:
                        self.liststore[rownum][0] = newname
        nbook = self.wtree.get_widget("notebook1")
        tab = nbook.get_tab_label(nbook.get_nth_page(nbook.get_current_page()))
        label = tab.get_text()
        progressbar = self.wtree.get_widget("progressbar1")
        progressbar.set_fraction(float(percentage) / 100)
        progressbar.set_text(u"%s:   %s %%" % (label, percentage))
        while gtk.events_pending():
            gtk.main_iteration(False)
        return self._cancelled

    def on_cancel_activate(self, widget, *dummy):
        """
        Called from the cancel button.
        """
        logger = logging.getLogger("exigui.on_cancel_activate")
        self._cancelled = True
        widget.set_sensitive(False)
        logger.warning("CANCELLED!")

    def on_run_activate(self, widget, *dummy):
        """
        Called from the run button.
        """
        logger = logging.getLogger("exigui.on_run_activate")
        cancel_button = self.wtree.get_widget("cancel_button")
        cancel_button.set_sensitive(True)
        nbook = self.wtree.get_widget("notebook1")
        nbook.set_sensitive(False)
        widget.set_sensitive(False)
        self._cancelled = False

        tab = nbook.get_tab_label(nbook.get_nth_page(nbook.get_current_page()))
        label = tab.get_text()
        logger.warning("Running %s", label)
        method = getattr(self, "_run_" + label.replace(" ", "_"))
        method()

        progressbar = self.wtree.get_widget("progressbar1")
        progressbar.set_fraction(0 / 100)
        progressbar.set_text(u"")
        nbook.set_sensitive(True)
        cancel_button.set_sensitive(False)
        widget.set_sensitive(True)

    def _run_exiimport(self):
        """ Run exiimport. """
        logger = logging.getLogger("exigui.run_exiimport")
        args = ["-v"]
        import_dir = self.wtree.get_widget("exiimport_importdir_entry")
        target_dir = self.wtree.get_widget("exiimport_targetdir_entry")
        if import_dir.get_text():
            args.append("--mount=" + import_dir.get_text())
        if target_dir.get_text():
            args.append("--target=" + target_dir.get_text())
        try:
            exiimport.run(args, self._progress_callback)
        except IOError, msg:
            logger.error("ERROR: %s", msg)

    def _run_exirename(self):
        """ Run exirename. """
        logger = logging.getLogger("exigui.run_exirename")
        args = ["-v"]
        initials = self.wtree.get_widget("exirename_artist_initials_entry")
        cam_id = self.wtree.get_widget("exirename_cam_id_entry")
        if self._is_active("exirename_artist_initials_entry_button_custom"):
            args.append("--artist_initials=" + initials.get_text())
        if self._is_active("exirename_cam_id_entry_button_custom"):
            args.append("--cam_id=" + cam_id.get_text())
        if self._is_active("exirename_include_timestamp_checkbutton"):
            args.append("--with_time")
        args += [entry[0] for entry in self.liststore]
        try:
            exirename.run(args, self._progress_callback)
        except IOError, msg:
            logger.error("ERROR: %s", msg)

    def _run_exiperson(self):
        """ Run exiperson. """
        logger = logging.getLogger("exigui.run_exiperson")
        args = ["-v"]
        exif_section = self.wtree.get_widget("exiperson_section_entry")
        if self._is_active("exiperson_section_entry_button_custom"):
            args.append("--section=" + exif_section.get_text())
        args += [entry[0] for entry in self.liststore]
        try:
            exiperson.run(args, self._progress_callback)
        except IOError, msg:
            logger.error("ERROR: %s", msg)

    def _run_exiconvert(self):
        """ Run exiconvert. """
        logger = logging.getLogger("exigui.run_exiconvert")
        args = ["-v"]
        if self._is_active("exiconvert_remove_lqjpeg_checkbutton"):
            args.append("--remove-lqjpeg")
        args += [entry[0] for entry in self.liststore]
        try:
            exiconvert.run(args, self._progress_callback)
        except IOError, msg:
            logger.error("ERROR: %s", msg)

    def _run_exiassign(self):
        """ Run exiassign. """
        logger = logging.getLogger("exigui.run_exiassign")
        args = ["-v"]
        if self._is_active("exiassign_force_checkbutton"):
            args.append("--force")
        args += [entry[0] for entry in self.liststore]
        try:
            exiassign.run(args, self._progress_callback)
        except IOError, msg:
            logger.error("ERROR: %s", msg)

    def _run_exigate(self):
        """ Run exigate. """
        logger = logging.getLogger("exigui.run_exigate")
        if self._is_active("exigate_nooptions"):
            args = ["-v"]
        if self._is_active("exigate_addfields"):
            args = ["--additional-fields"]
        if self._is_active("exigate_template"):
            args = ["--template"]
        if self._is_active("exigate_cleanup"):
            args = ["--cleanup"]
        args.append("-v")
        args += [entry[0] for entry in self.liststore]
        try:
            exigate.run(args, self._progress_callback)
        except IOError, msg:
            logger.error("ERROR: %s", msg)

    def batch_run(self, *dummy):
        """ Run "scripts" in a batch without user interaction. """
        if len(self.batch_scripts) > 0:
            logger = logging.getLogger("exigui.batch_run")
            nbook = self.wtree.get_widget("notebook1")
            tabs = []
            for tabnum in range(0, nbook.get_n_pages()):
                tab_label = nbook.get_tab_label(nbook.get_nth_page(tabnum))
                tabs.append(tab_label.get_text())
            for script in self.batch_scripts:
                if script not in tabs:
                    logger.error("There is no %s tab available!", script)
                    return 1
            run_button = self.wtree.get_widget("activate_button")
            for script in self.batch_scripts:
                nbook.set_current_page(tabs.index(script))
                self.on_run_activate(run_button)
            self.batch_scripts = []
            if self.batch_tmpdir:
                self.set_text("exiimport_targetdir_entry", self.batch_target)
                for subdir in os.listdir(self.batch_tmpdir):
                    target_subdir = subdir
                    while os.path.exists(os.path.join(self.batch_target,
                                                      target_subdir)):
                        logger.warning("%s already exists!",
                                       os.path.join(self.batch_target,
                                                    target_subdir))
                        target_subdir += "+"
                    logger.info("Moving %s into target directory %s", subdir,
                                os.path.join(self.batch_target, target_subdir))
                    try:
                        shutil.move(os.path.join(self.batch_tmpdir, subdir),
                                    os.path.join(self.batch_target,
                                                 target_subdir))
                    except shutil.Error, msg:
                        logger.error("ERROR: %s", msg)
                    for rownum in range(0, len(self.liststore)):
                        self.liststore[rownum][0] = \
                            self.liststore[rownum][0].replace(
                                os.path.join(self.batch_tmpdir, subdir),
                                os.path.join(self.batch_target, target_subdir))

        return 0


def run(argv):
    """
    Take an equivalent of sys.argv[1:] and run the GUI.
    """
    parser = optparse.OptionParser()
    parser.set_defaults(batch_order="exiimport,exirename,exiperson,"
                                    "exiconvert,exiassign,exigate")
    parser.add_option("-m", "--mount", dest="mount",
                      help="Mountpoint of directory to import.")
    parser.add_option("-t", "--target", dest="target",
                      help="Target directory. A subdirectory will be created"
                      " in this directory.")
    parser.add_option("-d", "--device", dest="device",
                      help="(Ignored for backwards compatibility. Don't use.)")
    parser.add_option("--cam_id", "-c", dest="cam_id",
                      help="ID string for the camera model. Should normally be"
                      " three characters long.")
    parser.add_option("--artist_initials", "-a", dest="artist_initials",
                      help="Initials of the artist. Should be two characters"
                      " long.")
    parser.add_option("-T", "--with_time", action="store_true",
                      dest="with_time",
                      help="Create filenames containing the image time, for"
                      " example 20071231-235959-n001234-xy000.jpg instead"
                      " of 20071231-n001234-xy000.jpg .")
    parser.add_option("-b", "--batch", action="store_true", dest="batch",
                      help="Autorun from exiimport over exirename, exiperson "
                      "and exiconvert to exiassign.")
    parser.add_option("--batch_order",
                      help="Comma separated processing order for --batch. "
                      "Default: %default")
    parser.add_option("--batch_tmpdir",
                      help="If the processing order starts with 'exiimport', "
                      "this temporary directory is used for batch processing. "
                      "After batch processing, the created subdirectory will "
                      "be moved into the target directory.")
    parser.add_option("--nofork", action="store_true",
                      help="Do not fork, stay in foreground instead.")
    options, args = parser.parse_args(argv)

    if options.nofork or os.fork() == 0:
        win1 = Window1()
        if options.mount:
            win1.set_text("exiimport_importdir_entry", options.mount)
        if options.target:
            win1.set_text("exiimport_targetdir_entry", options.target)
        if options.cam_id:
            win1.set_active("exirename_cam_id_entry_button_custom")
            win1.set_text("exirename_cam_id_entry", options.cam_id)
        if options.artist_initials:
            win1.set_active("exirename_artist_initials_entry_button_custom")
            win1.set_text("exirename_artist_initials_entry",
                          options.artist_initials)
        if options.with_time:
            win1.set_active("exirename_include_timestamp_checkbutton")
        if len(args) > 0:
            win1.set_filelist(args)
        if options.batch:
            win1.batch_scripts = options.batch_order.split(',')
            if options.batch_tmpdir and "exiimport" in win1.batch_scripts:
                if not options.mount or not options.target:
                    print("Wrong syntax. Missing --mount or --target\n"
                          + parser.format_help())
                    win1.batch_scripts = []
                win1.batch_tmpdir = tempfile.mkdtemp(dir=options.batch_tmpdir)
                win1.batch_target = options.target
                win1.set_text("exiimport_targetdir_entry", win1.batch_tmpdir)

        gtk.main()

    return 0


if __name__ == "__main__":
    run(sys.argv[1:])



#!/usr/bin/python
# -*- coding: utf-8 -*-
# vim: tabstop=4 expandtab shiftwidth=4
"""
A module for reading and writing EXIF information.
"""
__revision__ = "$Id$"

import re
import locale
import logging
import subprocess


class Exif(object):
    """
    A class for reading and writing EXIF information.
    """

    def __init__(self, filename):
        """
        Create an Exif object for filename.
        """
        self.filename = filename
        self.fields = {}

    def read_exif(self):
        """
        Read EXIF information from self.filename into self.fields
        Raises IOError on errors.
        """
        exiftool = subprocess.Popen(["exiftool", "-d", "%s", "-S",
                                     self.filename],
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)
        stdout, stderr = exiftool.communicate()
        # exiftool doesn't necessarily use stderr on errors...
        for line in stdout.splitlines():
            if line.startswith("Error:"):
                stderr += line
        if len(stderr) > 0:
            raise IOError(stderr)
        else:
            for line in stdout.splitlines():
                if ": " in line:
                    key, value = line.split(": ", 1)
                    self.fields[key] = value.strip()
            # We have to read the ImageDescription binary because it may
            # contain things like line breaks.
            self.fields["ImageDescription"] = \
                "".join(subprocess.Popen(["exiftool", "-b",
                                          "-ImageDescription",
                                          self.filename],
                                         stdout=subprocess.PIPE).stdout)
            # We don't need an empty "ImageDescription" for any merge
            # operations.
            if self.fields["ImageDescription"] == "":
                del self.fields["ImageDescription"]
        # Be sure to have proper UTF8 strings
        for key in self.fields:
            self.fields[key] = self._decode(self.fields[key])

    @staticmethod
    def _decode(field):
        """
        Decode string "field" by trying these encodings:

        UTF-8, Current locale, Latin1

        If none of these succeeds, just despair and return "???".
        """
        encodings = ['utf-8']
        #
        # next we add anything we can learn from the locale
        try:
            encodings.append(locale.nl_langinfo(locale.CODESET))
        except AttributeError:
            pass
        try:
            encodings.append(locale.getlocale()[1])
        except (AttributeError, IndexError):
            pass
        try:
            encodings.append(locale.getdefaultlocale()[1])
        except (AttributeError, IndexError):
            pass
        #
        # we try 'latin-1' last
        encodings.append('latin-1')
        for enc in encodings:
            # some of the locale calls
            # may have returned None
            if not enc:
                continue
            try:
                decoded = unicode(field, enc)
            except (UnicodeError, LookupError):
                decoded = unicode("???", "utf-8")
            else:
                break

        return decoded

    def write_exif(self):
        """
        Write Exif Information from self.fields into self.filename.
        Raises IOError on errors.
        """
        logger = logging.getLogger("exif.write_exif")
        command = ["exiftool", "-overwrite_original", "-P"]
        for field in self.fields.keys():
            if field == "Keywords":
                for keyword in self.fields[field].split(","):
                    command.append("-%s=%s" % (field, keyword.strip()))
            elif field == "DateTimeOriginal":
                # TODO: Writing back DateTimeOriginal seems to be a bad idea
                # since gthumb drops seconds and there is some confusion with
                # time strings and epoch.
                continue
            else:
                command.append("-%s=%s" % (field, self.fields[field]))
        command.append(self.filename)
        logger.debug("Calling: %s", command)
        exiftool = subprocess.Popen(command, stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)
        # exiftool doesn't reflect errors in it's return code, so we have to
        # assume an error if something is written to stderr.
        errors = exiftool.stderr.readlines()
        if len(errors) > 0:
            raise IOError("".join(errors + exiftool.stdout.readlines()))
        return exiftool.wait()

    def update_exif(self, sourcefile):
        """
        Copy Exif Information from sourcefile into destfile and merge in values
        from myexif. The fields used for merging have to be defined in
        exiffields.
        Raises IOError on errors.
        """
        logger = logging.getLogger("exif.update_exif")
        # Fields we want to keep
        exiffields = ["Artist", "Credit", "Copyright", "CopyrightNotice",
                      "ImageDescription", "Keywords", "Location",
                      "UserComment", "XPTitle"]
        command = ["exiftool", "-overwrite_original", "-x", "Orientation",
                   "-P", "-TagsFromFile", sourcefile, "--Keywords"]
        for field in exiffields:
            if field in self.fields:
                if field == "Keywords":
                    for keyword in self.fields[field].split(","):
                        command.append("-%s=%s" % (field, keyword.strip()))
                else:
                    command.append("-%s=%s" % (field, self.fields[field]))
        command.append(self.filename)
        logger.debug("Calling: %s", command)
        exiftool = subprocess.Popen(command, stdout=subprocess.PIPE,
                                    stderr=subprocess.PIPE)
        # exiftool doesn't reflect errors in it's return code, so we have to
        # assume an error if something is written to stderr but ignore
        # warnings too.
        errors = exiftool.stderr.readlines()
        if len(errors) > 0:
            myregex = re.compile("^Warning: \\[minor\\] .* Fixed.")
            mymatch = myregex.match(errors[0])
            if mymatch:
                logger.info("%s", errors)
            else:
                raise IOError("".join(errors + exiftool.stdout.readlines()))
        return exiftool.wait()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# vim: tabstop=4 expandtab shiftwidth=4
"""
Handle reading and writing gthumb comment files.
"""
__revision__ = "$Id$"

import os
import re
import gzip
import xml.dom.minidom


class Gthumb(object):
    """
    A class that handles reading and writing gthumb comment files
    for an image file.
    """

    def __init__(self, filename):
        """
        Create a Gthumb object for image filename.
        """
        self.fields = {}
        self.filename = os.path.abspath(filename)
        self.commentsdir = os.path.join(os.path.dirname(self.filename),
                                        ".comments")
        self.commentsfile = os.path.join(self.commentsdir,
                                         os.path.basename(self.filename) +
                                         ".xml")

    def read(self):
        """
        Read gthumb comment information for filename and fill the dictionary
        self.fields with the corresponding EXIF values.
        Return True on success and False if no comment file exists.

        Translation of Exif values to gthumb sections:

            "Artist"            => "Note" line: "^Artist::.*"
            "Credit"            => "Note" line: "^Credit::.*"
            "Copyright"         => "Note" line: "^Copyright::.*"
            "CopyrightNotice"   => "Note" line: "^CopyrightNotice::.*"
            "UserComment"       => "Note" line: "^UserComment::.*"
            "ImageDescription"  => "Note" free text without "^\\w+::"
            "Keywords"          => "Keywords" comma separated list
            "Location"          => "Place"
            "DateTimeOriginal"  => "Time" in seconds since 1970-01-01
            "XPTitle"           => First line of "Note", if any

        In fact, any line of "Note" that consists of a keyword followed by
        two colons and some random text is converted into it's EXIF equivalent.
        """
        gthumb_to_exif = {"Time": "DateTimeOriginal",
                          "Place": "Location",
                          "Keywords": "Keywords"}

        if os.path.isfile(self.commentsfile):
            mydata = {}
            mydom = xml.dom.minidom.parse(gzip.open(self.commentsfile))
            for field in ("Note", "Keywords", "Place", "Time"):
                mynodes = mydom.getElementsByTagName(field)
                if len(mynodes) > 0:
                    mynodes = mynodes[0].childNodes
                    if len(mynodes) > 0:
                        mydata[field] = mynodes[0].wholeText

            for key in gthumb_to_exif:
                if key in mydata:
                    self.fields[gthumb_to_exif[key]] = mydata[key]
            if "Note" in mydata:
                note = []
                myregex = re.compile("(\\w+)::(.*)$")
                for line in mydata["Note"].split("\n"):
                    match = myregex.match(line)
                    if match:
                        self.fields[match.group(1)] = match.group(2).strip()
                    else:
                        note.append(line)
                if len(note) > 0:
                    self.fields["XPTitle"] = note[0]
                    self.fields["ImageDescription"] = "\n".join(note).strip()
            return True
        else:
            return False

    def write(self, myaddfields=False, mytemplate=False):
        """
        Write Exif information from self.fields into gthumb comment file.
        Apply reverse mapping of self.read().

        myaddfields: Write some additional fields.
        mytemplate: Write empty fields as well.
        """
        exiffields = ["Artist", "Credit", "Copyright", "CopyrightNotice",
                      "UserComment"]
        mydata = {}
        mydata["Place"] = self.fields.get("Location", "")
        mydata["Time"] = self.fields.get("DateTimeOriginal", "")
        mydata["Note"] = self.fields.get("ImageDescription", "")
        tmpkeywords = []
        for keyword in self.fields.get("Keywords", "").split(","):
            tmpkeywords.append(keyword.strip())
        mydata["Keywords"] = ",".join(tmpkeywords)

        # Add XPTitle to Note if Note doesn't contain it already
        mytitle = self.fields.get("XPTitle", "")
        if len(mytitle) > 0 and not mytitle in mydata["Note"].splitlines():
            mydata["Note"] = self.fields["XPTitle"] + "\n" + mydata["Note"]
        if myaddfields or mytemplate:
            mydata["Note"] += "\n"
            for field in exiffields:
                fieldvalue = self.fields.get(field, "")
                if mytemplate or len(fieldvalue) > 0:
                    mydata["Note"] += "\n" + field + ":: " + fieldvalue

        mydom = xml.dom.minidom.Document()
        comment = mydom.createElement("Comment")
        comment.setAttribute("format", "2.0")
        mydom.appendChild(comment)

        for name in ("Place", "Time", "Note", "Keywords"):
            element = mydom.createElement(name)
            text = mydom.createTextNode(mydata[name])
            element.appendChild(text)
            comment.appendChild(element)

        if not os.path.isdir(self.commentsdir):
            os.makedirs(self.commentsdir)
        gzip.open(self.commentsfile, "wb").write(mydom.toxml(encoding="utf-8"))

    def get_mtime(self):
        """
        Get modification time of comment file or 0 if it doesn't exist.
        """
        if os.path.isfile(self.commentsfile):
            return os.path.getmtime(self.commentsfile)
        else:
            return 0

    def set_mtime(self, mtime):
        """
        Set modification time of comment file to mtime.
        Returns True on success and False if no comment file exists.
        """
        if os.path.isfile(self.commentsfile):
            os.utime(self.commentsfile, (mtime, mtime))
            return True
        else:
            return False

    def cleanup(self):
        """
        Remove additional fields from gthumb comments.
        (other fields than Note, Keywords, Place, Time)
        Return True if a file is cleaned, False otherwise.
        """
        if self.read():
            for field in self.fields:
                if field not in ("ImageDescription", "Location",
                                 "DateTimeOriginal", "Keywords", "XPTitle"):
                    mtime = self.get_mtime()
                    self.write()
                    self.set_mtime(mtime)
                    return True
        return False



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim: tabstop=4 expandtab shiftwidth=4
"""
Unit tests for exirename.py
"""
__revision__ = "$Id$"


import shutil
import os
import sys
import tempfile
import time
import unittest

import NikonD70
sys.path.append(os.path.dirname(sys.path[0]))
import exiflow.configfile
import exiflow.exirename
import exiflow.filelist


class TestExirename(unittest.TestCase):
    """ Tests for exirename.py """

    def setUp(self):
        """ Create a directory with an image. """
        self.__data_dir = os.path.join(os.path.dirname(__file__), "testdata")
        self.__tempdir = tempfile.mkdtemp()
        self.__configdir = tempfile.mkdtemp()
        exiflow.configfile.global_config_dir = self.__configdir
        exiflow.configfile.local_config_dir = self.__configdir

    def tearDown(self):
        """ Clean up. """
        shutil.rmtree(self.__tempdir)
        shutil.rmtree(self.__configdir)

    def test_run__space_in_filename(self):
        """ Test run() with file containing a space and "--with_time". """
        shutil.copyfile(os.path.join(self.__data_dir, "NikonD70.jpg"),
                        os.path.join(self.__tempdir, "Nikon D70.jpg"))
        exiflow.exirename.run(argv=["--cam_id=n00", "--artist_initials=xy",
                              "--with_time", "--verbose", self.__tempdir])
        self.assertEqual(os.listdir(self.__tempdir),
                         ["20040330-104346-n000070-xy000.jpg"])

    def test_run__low_quality_jpg(self):
        """ Test run() with a low quality JPG preview. """
        shutil.copyfile(os.path.join(self.__data_dir, "python.jpg"),
                        os.path.join(self.__tempdir, "DSC1234.jpg"))
        shutil.copyfile(os.path.join(self.__data_dir, "python.jpg"),
                        os.path.join(self.__tempdir, "DSC1234.nef"))
        exiflow.exirename.run(argv=["--cam_id=n00", "--artist_initials=xy",
                              self.__tempdir])
        self.assertTrue(time.strftime("%Y%m%d") + "-n001234-xy00l.jpg" in
                        os.listdir(self.__tempdir))

    def test_run__no_number_in_filename(self):
        """ Test run() with a filename not containing a number. """
        shutil.copyfile(os.path.join(self.__data_dir, "python.jpg"),
                        os.path.join(self.__tempdir, "python.jpg"))
        exiflow.exirename.run(argv=["--cam_id=n00", "--artist_initials=xy",
                              self.__tempdir])
        # Nothing should have changed.
        self.assertEqual(os.listdir(self.__tempdir), ["python.jpg"])

    def test_run__no_camid_no_artistinitials(self):
        """ Test run() with undefined cam_id and artist_initials. """
        shutil.copyfile(os.path.join(self.__data_dir, "python.jpg"),
                        os.path.join(self.__tempdir, "DSC1234.jpg"))
        exiflow.exirename.run(argv=[self.__tempdir])
        # Nothing should have changed.
        self.assertEqual(os.listdir(self.__tempdir), ["DSC1234.jpg"])

    def test_run__already_in_exiflow_format(self):
        """ Test run() with a filename already in our naming scheme. """
        shutil.copyfile(os.path.join(self.__data_dir, "NikonD70.jpg"),
                        os.path.join(self.__tempdir,
                                     "20121212-n000001-ad000.jpg"))
        exiflow.exirename.run(argv=["--with_time", self.__tempdir])
        self.assertEqual(os.listdir(self.__tempdir),
                         ["20121212-104346-n000001-ad000.jpg"])
        
    def test_run__callback(self):
        """ Test run() with a callback function. """
        mylist = []

        def callback(*dummy_args):
            """ Just set callback_called to True. """
            mylist.append(True)
            return True

        shutil.copyfile(os.path.join(self.__data_dir, "python.jpg"),
                        os.path.join(self.__tempdir, "python.jpg"))
        exiflow.exirename.run(argv=["--cam_id=n00", "--artist_initials=xy",
                              self.__tempdir], callback=callback)
        self.assertEqual(mylist, [True])

    def test_rename_file__exceptions(self):
        """ Test exceptions of rename_file(). """
        my_filelist = exiflow.filelist.Filelist([])
        jpg_file = os.path.join(self.__tempdir, "20121212-n000001-ad000.jpg")
        shutil.copyfile(os.path.join(self.__data_dir, "NikonD70.jpg"), jpg_file)
        # File name doesn't change.
        self.assertRaises(IOError, exiflow.exirename.rename_file, jpg_file,
                          my_filelist, with_time=False)
        # Target already exists.
        open(os.path.join(self.__tempdir, "20121212-104346-n000001-ad000.jpg"),
             'w').write(":-)")
        self.assertRaises(IOError, exiflow.exirename.rename_file, jpg_file,
                          my_filelist, with_time=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Install the exiflow collection on your computer.
"""
__revision__ = "$Id: $"

#from distutils.core import setup
from setuptools import setup

setup(name='Exiflow',
      install_requires=["glade2"],
      version='0.4.5.16',
      packages=['exiflow'],
      scripts=["exiassign", "exiconvert", "exigate", "exigui",
               "exiimport", "exiperson", "exirename"],
      package_data={"exiflow": ["exigui.glade"]},
      data_files=[('share/doc/exiflow', ['README', 'INSTALL', 'COPYING'])],
#      entry_points={'console_scripts': ["exiassign = exiflow.exiassign:run",
#                                        "exiconvert = exiflow.exiconvert:run",
#                                        "exigate = exiflow.exigate:run",
#                                        "exiimport = exiflow.exiimport:run",
#                                        "exiperson = exiflow.exiperson:run",
#                                        "exirename = exiflow.exirename:run"],
#                    'gui_scripts': ["exigui = exiflow.exigui:run"]},
      author='Ulf Rompe, Sebastian Berthold',
      author_email='exiflow-devel@lists.sourceforge.net',
      license='GNU General Public License (GPL)',
      platforms="any",
      url='http://exiflow.org/',
      description='A set of tools including a little GUI to '
                  'provide a complete digital photo workflow '
                  'for Unixes, using EXIF headers as the '
                  'central information repository.',
      long_description='A set of tools including a little GUI '
                       'to provide a complete digital photo '
                       'workflow for Unixes. EXIF headers are '
                       'used as the central information '
                       'repository, so users may change their '
                       'software at any time without loosing '
                       'any data. The tools may be used '
                       'individually or combined.',
      download_url='http://sourceforge.net/project/showfiles.php?group_id=151136',
      classifiers=['Development Status :: 4 - Beta',
                   'Environment :: Console',
                   'Environment :: X11 Applications :: GTK',
                   'Intended Audience :: End Users/Desktop',
                   'License :: OSI Approved :: GNU General Public License (GPL)',
                   'Operating System :: POSIX',
                   'Operating System :: POSIX :: Linux',
                   'Operating System :: Unix',
                   'Programming Language :: Python',
                   'Topic :: Multimedia :: Graphics',
                   'Topic :: Multimedia :: Graphics :: Graphics Conversion',
                   'Topic :: System :: Archiving',
                   'Topic :: Utilities'])



#!/usr/bin/python
# -*- coding: utf-8 -*-
# vim: tabstop=4 expandtab shiftwidth=4
"""
Personalize images by setting EXIF fields to values spezified in exif.cfg or
on command line. Any arbitrary field name may be configured or given.

This is normally used for artist and copyright information.

Field names are the short versions supported by Exiftool. Look up the
Exiftool documentation for information about possible names.
"""

__revision__ = "$Id$"

import sys
import logging
import optparse
from . import exif
from . import filelist
from . import configfile


def personalize_file(filename, personals, options_section_personals,
                     forced_personals):
    """
    Personalize an image using data from the dictionary "personals".
    The optional "forced_personals" override all other personals.
    """
    logger = logging.getLogger("exiperson.personalize_file")
    exifconfig = configfile.parse("exif")
    exif_file = exif.Exif(filename)
    try:
        exif_file.read_exif()
    except IOError, msg:
        logger.warning("Skipping %s: %s", filename, msg)
        return 1

    if "Model" in exif_file.fields:
        if exifconfig.has_section(exif_file.fields["Model"]):
            personals += exifconfig.items(exif_file.fields["Model"])
        else:
            configfile.append("exif", exif_file.fields["Model"],
                              ("artist", "contact"))
            sys.stderr.write("Get rid of this message by defining at least"
                             " an empty [%s] section.\n" %
                             exif_file.fields["Model"])

    if len(options_section_personals) > 0:
        personals += options_section_personals

    if len(personals) == 0 and len(forced_personals) == 0:
        logger.warning("No [all] or [%s] section with data, nor custom tags,"
                       " skipping.", exif_file.fields["Model"])
        return 1

    exif_file.fields = {}
    for key, value in personals:
        exif_file.fields[key] = value

    if len(forced_personals) > 0:
        for key in forced_personals:
            exif_file.fields[key] = forced_personals[key]

    try:
        exif_file.write_exif()
    except IOError, msg:
        logger.error("Error writing EXIF data:%s\n%s", filename, msg)
        return 1
    return 0


def run(argv, callback=None):
    """
    Take an equivalent of sys.argv[1:] and optionally a callable.
    Parse options, personalize files and optionally call the callable
    on every processed file with 3 arguments: filename, newname, percentage.
    If the callable returns True, stop the processing.
    """
    parser = optparse.OptionParser(usage="usage: %prog [options] "
                                         "[-- -TAGNAME=VALUE [...]] "
                                         "<files or dirs>")
    parser.description = ("Hint: %prog is able to set or unset any tags "
                          "supported in ExifTool with -TAGNAME=[VALUE] "
                          "syntax.")
    parser.add_option("--section", "-s", dest="section",
                      help="Name of a config file section to be read. This is"
                           " useful if different people are using the same"
                           " camera model. By default, the section name is"
                           " guessed from the camera model. Section 'all' is"
                           " the default.")
    parser.add_option("-v", "--verbose", action="store_true", dest="verbose",
                      help="Be verbose.")
    options, args = parser.parse_args(argv)

    logging.basicConfig(format="%(module)s: %(message)s")
    if options.verbose:
        logging.getLogger().setLevel(logging.INFO)
    logger = logging.getLogger("exiperson")

    exifconfig = configfile.parse("exif")

    defaultpersonals = []
    options_section_personals = []
    if exifconfig.has_section("all"):
        defaultpersonals += exifconfig.items("all")

    if options.section:
        if exifconfig.has_section(options.section):
            options_section_personals += exifconfig.items(options.section)
        else:
            logger.error("ERROR: Section %s not found in config files",
                         options.section)
            sys.exit(1)

    # collect EXIF data supplied on command line
    forced_personals = {}
    remaining_args = []
    for arg in args:
        if arg.startswith("-") and "=" in arg:
            forced_personals.update(dict((arg.lstrip("-").split("="),)))
        else:
            remaining_args.append(arg)

    for filename, percentage in filelist.Filelist(remaining_args):
        logger.info("%3s%% %s", percentage, filename)
        if callable(callback):
            if callback(filename, filename, percentage):
                break
        # Note to programmer:
        # The [:] is needed to get a slice copy instead of a reference.
        personalize_file(filename, defaultpersonals[:],
                         options_section_personals[:], forced_personals)


if __name__ == "__main__":
    run(sys.argv[1:])
